import abc
from typing import TYPE_CHECKING
from typing import Any
from typing import Dict
from typing import Optional
from typing import Tuple

from report_writer.core.report_objects.cursor import Cursor
from report_writer.impl.xlsx.report.report_file import ReportFile


if TYPE_CHECKING:
    from report_writer.core.content.base import BaseContent
    from report_writer.core.report_objects.base import BaseReportObject
    from report_writer.core.report_objects.cursor import CursorFrame


class BaseReportBuilder(metaclass=abc.ABCMeta):
    """
    Базовый построитель отчёта
    """
    def __init__(self,
                 path: str,
                 data: Any
                 ) -> None:
        self._path = path
        self._data = data

    @abc.abstractmethod
    def make(self
             ) -> None:
        """
        Метод выполняющий создание отчёта
        """
        raise NotImplementedError


class ReportObjectsBuilder(BaseReportBuilder, metaclass=abc.ABCMeta):
    """
    Построитель отчёта из объектов отчёта

    Добавляет методы для создания объектов отчёта и их записи
    """
    def __init__(self,
                 path: str,
                 data: Any
                 ) -> None:
        super(ReportObjectsBuilder, self).__init__(path, data)
        self._report_objects: Dict[
            str, Tuple['BaseReportObject', 'BaseContent']] = {}
        self._init_report_objects()

    def _add_report_object(self,
                           name: str,
                           report_object: 'BaseReportObject',
                           content: 'BaseContent'
                           ) -> None:
        """
        Добавление объекта отчёта
        """
        self._report_objects[name] = (report_object, content)

    def _get_report_object(self,
                           name: str
                           ) -> Tuple['BaseReportObject', 'BaseContent']:
        """
        Получение объекта отчёта
        """
        return self._report_objects[name]

    def _write_report_object(self,
                             name: str,
                             cursor: Cursor,
                             direction: Tuple[int, int] = Cursor.VERTICAL
                             ) -> 'CursorFrame':
        """
        Запись объекта отчёта
        """
        report_object, content = self._get_report_object(name)

        with cursor.get_state() as state:
            report_object.write_object(
                cursor, content.render(self._data), direction)

        return state.frame

    def _init_report_objects(self
                             ) -> None:
        """
        Инициализация объектов отчёта
        """
        pass  # pragma: no cover


class XlsxReportBuilder(ReportObjectsBuilder, metaclass=abc.ABCMeta):
    """
    Построитель xlsx отчётов
    """

    # Путь до шаблона отчёта, если не указан - шаблон не будет использоваться
    template_path: Optional[str] = None

    def __init__(self,
                 path: str,
                 data: Any
                 ) -> None:
        self._report = self._get_report_file(path, data)
        super(XlsxReportBuilder, self).__init__(path, data)

    def _get_template_context(self,
                              data: Any
                              ) -> Optional[dict]:
        """
        Получение контекста для шаблона отчёта
        """
        pass  # pragma: no cover

    def _get_report_file(self,
                         path: str,
                         data: Any
                         ) -> ReportFile:
        """
        Получение объекта файла отчёта
        """
        return ReportFile(
            path,
            self.template_path,
            self._get_template_context(data)
        )

    @abc.abstractmethod
    def _build(self
               ) -> None:
        """
        Построение отчёта
        """
        raise NotImplementedError

    def make(self
             ) -> None:
        self._build()
        self._report.save()
