import os
import uuid
from typing import TYPE_CHECKING
from typing import Any
from typing import Optional
from typing import Tuple
from typing import Type


if TYPE_CHECKING:
    from m3_report_writer.builders import BaseReportBuilder


class BaseReporter:
    """
    Базовый составитель отчёта
    """
    reports_dir: str = ''  # Папка с отчётами
    reports_url: str = ''  # URL для отчётов

    extension: str = '.xlsx'
    report_builder_cls: Optional[Type['BaseReportBuilder']] = None

    def _get_report_data(self,
                         params: Any
                         ) -> Any:
        """
        Получение данных отчёта из параметров отчёта
        """
        return params

    def _get_report_builder(self,
                            path: str,
                            data: Any
                            ) -> 'BaseReportBuilder':
        """
        Получение построителя отчёта
        """
        if self.report_builder_cls is None:
            raise NotImplementedError

        return self.report_builder_cls(path, data)

    def _create_report_name(self
                            ) -> str:
        """
        Генерация имени отчёта
        """
        return str(uuid.uuid4())[0:16] + self.extension

    def _get_report_path(self
                         ) -> Tuple[str, str]:
        """
        Получение пути до файла отчёта (в файловой системе и URL)
        """
        base_name = self._create_report_name()

        file_path = os.path.join(self.reports_dir, base_name)
        file_url = '/'.join((self.reports_url, base_name))

        return file_path, file_url

    def make_report(self,
                    path: str,
                    params: Any
                    ) -> None:
        """
        Построение отчёта
        """
        report_data = self._get_report_data(params)
        builder = self._get_report_builder(path, report_data)
        builder.make()

    def get_report(self,
                   params: Any
                   ) -> Tuple[str, str]:
        """
        Получение пути до готового отчёта
        """
        file_path, file_url = self._get_report_path()
        self.make_report(file_path, params)

        return file_path, file_url
