from typing import TYPE_CHECKING
from typing import Any
from typing import Dict
from typing import Optional
from typing import Type

from xlsxwriter import Workbook


if TYPE_CHECKING:
    from xlsxwriter.format import Format
    from xlsxwriter.worksheet import Worksheet


class ReportFile(object):
    """
    Файл отчёта

    :param path: Путь до файла отчёта
    :param options: Дополнительные настройки файла отчёта
    """
    def __init__(self,
                 path: str,
                 options: Optional[dict] = None
                 ) -> None:
        self._path = path
        self._workbook: Optional[Workbook] = None
        self._workbook_options: Optional[dict] = options
        self._active_worksheet: Optional['Worksheet'] = None
        self._worksheets: Dict[str, 'Worksheet'] = {}
        self._formats: Dict[str, 'Format'] = {}

    def _get_workbook(self
                      ) -> Workbook:
        """
        :return: Чистый Workbook
        """
        return Workbook(self._path, self._workbook_options)

    def _init_workbook(self
                       ) -> None:
        """
        Инициализация Workbook
        """
        self._workbook = self._get_workbook()

    @property
    def workbook(self
                 ) -> Workbook:
        """
        :return: Workbook файла отчёта
        """
        if self._workbook is None:
            self._init_workbook()

        return self._workbook

    @property
    def worksheet(self
                  ) -> 'Worksheet':
        """
        :return: Текущий активный лист
        """
        if self._active_worksheet is None:
            raise AttributeError('Не выбран активный лист')
        return self._active_worksheet

    def add_worksheet(self,
                      name: str,
                      title: str = None,
                      worksheet_cls: Type['Worksheet'] = None
                      ) -> None:
        """
        Добавить лист в файл отчёта

        :param name: Название
        :param title: Заголовок листа
        :param worksheet_cls: Класс листа
        """
        self._worksheets[name] = self.workbook.add_worksheet(
            name=title, worksheet_class=worksheet_cls)

        if self._active_worksheet is None:
            self.set_worksheet(name)

    def get_worksheet(self,
                      name: str
                      ) -> 'Worksheet':
        """
        :param name: Название листа
        :return: Лист отчёта
        """
        return self._worksheets[name]

    def set_worksheet(self,
                      name: str
                      ) -> 'Worksheet':
        """
        Сменить активный лист отчёта

        :param name: Название листа
        :return: Лист отчёта
        """
        self._active_worksheet = self.get_worksheet(name)
        return self._active_worksheet

    def add_format(self,
                   name: str,
                   **params: Any
                   ) -> 'Format':
        """
        Добавить именованный формат к файлу отчёта

        :param name: Наименование формата
        :param params: Параметры формата
        :return: Созданный формат
        """
        self._formats[name] = self.workbook.add_format(params)
        return self._formats[name]

    def get_format(self,
                   name: str
                   ) -> 'Format':
        """
        :param name: Название формата
        :return: Формат по названию
        """
        return self._formats[name]

    def save(self
             ) -> None:
        self.workbook.close()
