import os
from tempfile import NamedTemporaryFile
from unittest import TestCase

from openpyxl import load_workbook
from report_writer.core.report_objects.cursor import Cursor
from report_writer.core.report_objects.samples.simple import Cell
from report_writer.utils.read_utils import read_xlsx

from report_xlsxwriter.report_file import ReportFile
from report_xlsxwriter.writers.format import ColumnWidthWriter
from report_xlsxwriter.writers.format import ColumnWriter
from report_xlsxwriter.writers.format import FormattedCellWriter
from report_xlsxwriter.writers.format import FormattedMergedCellWriter
from report_xlsxwriter.writers.format import FormatWriter
from report_xlsxwriter.writers.format import RowHeightWriter
from report_xlsxwriter.writers.format import RowWriter
from report_xlsxwriter.writers.merge import MergedCellWriter
from report_xlsxwriter.writers.simple import CellWriter


# На Windows файл нельзя будет удалить
_DELETE_TMP_FILE = os.name != 'nt'


class XlsxwriterReportFileTest(TestCase):
    def test_report_file_saving(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_worksheet('main')
            cell = Cell(CellWriter(report))
            cursor = Cursor(0, 0)
            cell.write_object(cursor, 1, Cursor.VERTICAL)
            report.save()
            wb = load_workbook(tmp.name)
            self.assertEqual(next(read_xlsx(wb)), [[1]])

    def test_report_file_add_format(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_format('foo', bold=True)

            self.assertIn('foo', report._formats)
            self.assertEqual(report._formats['foo'].bold, True)
            report.save()

    def test_report_set_worksheet(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)

            with self.assertRaises(AttributeError):
                _ = report.worksheet

            report.add_worksheet('a', 'a')
            worksheet_a = report.get_worksheet('a')
            self.assertIs(report.worksheet, worksheet_a)
            report.add_worksheet('b', 'b')
            worksheet_b = report.get_worksheet('b')
            self.assertIs(report.worksheet, worksheet_a)
            report.add_worksheet('c', 'c')
            worksheet_c = report.get_worksheet('c')

            self.assertEqual(worksheet_a.name, 'a')
            self.assertEqual(worksheet_b.name, 'b')
            self.assertEqual(worksheet_c.name, 'c')

            report.set_worksheet('a')
            self.assertIs(report.worksheet, worksheet_a)
            report.set_worksheet('b')
            self.assertIs(report.worksheet, worksheet_b)
            report.set_worksheet('c')
            self.assertIs(report.worksheet, worksheet_c)

            cell = Cell(CellWriter(report))
            cursor = Cursor(0, 0)
            cell.write_object(cursor, 1, Cursor.VERTICAL)

            report.save()

            wb = load_workbook(tmp.name)
            self.assertEqual(list(read_xlsx(wb)), [
                [],
                [],
                [[1]],
            ])


class XlsxwriterReportObjectsTest(TestCase):
    def test_cell_object(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_worksheet('main')
            cursor = Cursor(0, 0)
            cell = Cell(CellWriter(report))

            cell.write_object(cursor, 1, Cursor.VERTICAL)

            report.save()
            wb = load_workbook(tmp.name)
            self.assertEqual(next(read_xlsx(wb)), [[1]])

        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_worksheet('main')
            cursor = Cursor(0, 0)
            cell = Cell(CellWriter(report))

            cell.write_object(cursor, 1, Cursor.VERTICAL)
            cell.write_object(cursor, 2, Cursor.VERTICAL)
            cell.write_object(cursor, 3, Cursor.HORIZONTAL)
            cell.write_object(cursor, 4, Cursor.HORIZONTAL)
            cell.write_object(cursor, 5, Cursor.HORIZONTAL)

            report.save()
            wb = load_workbook(tmp.name)

            self.assertEqual(
                next(read_xlsx(wb)),
                [
                    [1, None, None],
                    [2, None, None],
                    [3, 4,    5]
                ]
            )

    def test_merged_cell_object(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_worksheet('main')
            cursor = Cursor(0, 0)
            cell = Cell(MergedCellWriter(report, 2, 3), 2, 3)

            cell.write_object(cursor, 1, Cursor.VERTICAL)
            cell.write_object(cursor, 2, Cursor.HORIZONTAL)
            cell.write_object(cursor, 3, Cursor.HORIZONTAL)

            report.save()
            wb = load_workbook(tmp.name)
            self.assertEqual(
                next(read_xlsx(wb)),
                [
                    [1,    None, None, None, None, None],
                    [None, None, None, None, None, None],
                    [2,    None, None, 3,    None, None],
                    [None, None, None, None, None, None],
                ]
            )

    def test_format_writer(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            FormatWriter(report, 'foo', bold=True).write()

            self.assertIn('foo', report._formats)
            self.assertEqual(report._formats['foo'].bold, True)
            report.save()

    def test_formatted_cell(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_worksheet('main')
            cursor = Cursor(0, 0)
            cell = Cell(FormattedCellWriter(report, 'foo'))
            format_writer = FormatWriter(report, 'foo', bold=True)

            format_writer.write()
            cell.write_object(cursor, 0, Cursor.VERTICAL)

            report.save()

            wb = load_workbook(tmp.name)
            self.assertEqual(wb.active['A1'].font.b, True)

    def test_formatted_merged_cell(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_worksheet('main')
            cursor = Cursor(0, 0)
            cell = Cell(FormattedMergedCellWriter(report, 2, 3, 'foo'), 2, 3)
            format_writer = FormatWriter(report, 'foo', bold=True)

            format_writer.write()
            cell.write_object(cursor, 0, Cursor.VERTICAL)

            report.save()

            wb = load_workbook(tmp.name)
            self.assertEqual(wb.active['A1'].font.b, True)
            self.assertEqual(
                next(read_xlsx(wb)),
                [
                    [0,    None, None],
                    [None, None, None],
                ]
            )

    def test_column_format(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_worksheet('main')
            FormatWriter(report, 'column', bold=True).write()
            ColumnWriter(report, {'cell_format': 'column'}).write(0, 0, None)
            report.save()
            wb = load_workbook(tmp.name)
            self.assertTrue(wb.active.column_dimensions['A'].font.b)

    def test_row_format(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_worksheet('main')
            FormatWriter(report, 'row', bold=True).write()
            RowWriter(report, {'cell_format': 'row'}).write(0, 0, None)
            report.save()
            wb = load_workbook(tmp.name)
            self.assertTrue(wb.active.row_dimensions[1].font.b)

    def test_empty_row_col(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_worksheet('main')
            RowWriter(report).write(0, 0, None)
            report.save()

    def test_column_width_writer(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_worksheet('main')
            ColumnWidthWriter(report, 1).write(0, 0, 10)
            ColumnWidthWriter(report, 4).write(0, 2, 15)

            report.save()
            wb = load_workbook(tmp.name)
            self.assertEqual(wb.active.column_dimensions['A'].min, 1)
            self.assertAlmostEqual(
                # Библиотека прибавляет немного к ширине, т.к. excel
                # потом срезает это обратно
                wb.active.column_dimensions['A'].width, 10, delta=1)
            self.assertEqual(wb.active.column_dimensions['C'].min, 3)
            self.assertEqual(wb.active.column_dimensions['C'].max, 6)
            self.assertAlmostEqual(
                wb.active.column_dimensions['C'].width, 15, delta=1)

    def test_row_height_writer(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            report = ReportFile(tmp.name)
            report.add_worksheet('main')
            RowHeightWriter(report).write(0, 0, 100)

            report.save()
            wb = load_workbook(tmp.name)
            self.assertEqual(wb.active.row_dimensions[1].r, 1)
            self.assertEqual(wb.active.row_dimensions[1].height, 100)
