import abc
from typing import TYPE_CHECKING
from typing import Any
from typing import Dict
from typing import Optional

from report_xlsxwriter.writers.base import BaseReportWriter
from report_xlsxwriter.writers.merge import MergedCellWriter
from report_xlsxwriter.writers.simple import CellWriter


if TYPE_CHECKING:
    from report_xlsxwriter.report_file import ReportFile


class FormatWriter(BaseReportWriter):
    """
    Писатель для именованного стиля.

    :param report: Файл отчёта
    :param name: Название формата
    :param format_params: Параметры формата
    """
    def __init__(self,
                 report: 'ReportFile',
                 name: str,
                 **format_params: Any
                 ) -> None:
        super(FormatWriter, self).__init__(report)
        self._format_name: str = name
        self._format_params: Dict[str, Any] = format_params

    def write(self,
              row: int = 0,
              col: int = 0,
              data: Any = None
              ) -> None:
        self._report.add_format(self._format_name, **self._format_params)


class FormattedCellWriter(CellWriter):
    """
    Писатель форматированной клетки отчёта

    :param report: Файл отчёта
    :param format_name: Название формата
    """
    def __init__(self,
                 report: 'ReportFile',
                 format_name: str
                 ) -> None:
        super(FormattedCellWriter, self).__init__(report)
        self._format_name: str = format_name

    def write(self,
              row: int,
              col: int,
              data: Any
              ) -> None:
        self._report.worksheet.write(
            row, col, data,
            self._report.get_format(self._format_name))


class FormattedMergedCellWriter(MergedCellWriter):
    """
    Писатель форматированной объединённой клетки отчёта

    :param report: Файл отчёта
    :param height: Высота объединения
    :param width: Ширина объединения
    :param format_name: Название формата
    """
    def __init__(self,
                 report: 'ReportFile',
                 height: int,
                 width: int,
                 format_name: str
                 ) -> None:
        super(FormattedMergedCellWriter, self).__init__(report, height, width)
        self._format_name: str = format_name

    def write(self,
              row: int,
              col: int,
              data: Any
              ) -> None:
        self._report.worksheet.merge_range(
            row, col, row + self._height - 1, col + self._width - 1, data,
            self._report.get_format(self._format_name))


class TableFormatWriter(BaseReportWriter, metaclass=abc.ABCMeta):
    """
    Базовый писатель форматов для крупных элментов таблицы (строки/колонки)

    :param report: Файл отчёта
    :param opts: Опции формата
    """
    def __init__(self,
                 report: 'ReportFile',
                 opts: Optional[dict] = None,
                 ) -> None:
        super(TableFormatWriter, self).__init__(report)

        _opts: dict = opts or {}

        if 'cell_format' in _opts:
            _opts['cell_format'] = self._report.get_format(
                _opts['cell_format'])

        self._opts: Dict[str, Any] = {k: v for k, v in _opts.items() if k in (
            'width',
            'height',
            'cell_format',
            'options',
        )}


class ColumnWriter(TableFormatWriter):
    """
    Писатель формата колонки

    :param report: Файл отчёта
    :param opts: Опции формата
    :param cols: Сколько колонок охватывает писатель
    """
    def __init__(self,
                 report: 'ReportFile',
                 opts: Optional[dict] = None,
                 cols: int = 1,
                 ) -> None:
        super(ColumnWriter, self).__init__(report, opts)
        self._cols: int = cols

    def write(self,
              row: int,
              col: int,
              data: Any
              ) -> None:
        self._report.worksheet.set_column(
            col, col + self._cols - 1, **self._opts)


class RowWriter(TableFormatWriter):
    """
    Писатель формата строки
    """
    def write(self,
              row: int,
              col: int,
              data: Any
              ) -> None:
        self._report.worksheet.set_row(row, **self._opts)


class ColumnWidthWriter(BaseReportWriter):
    """
    Писатель ширины колонки.

    :param report: Файл отчёта
    :param cols: Сколько колонок охватывает писатель
    """
    def __init__(self,
                 report: 'ReportFile',
                 cols: int = 1,
                 ) -> None:
        super(ColumnWidthWriter, self).__init__(report)
        self._cols: int = cols

    def write(self,
              row: int,
              col: int,
              data: Any
              ) -> None:
        ColumnWriter(
            self._report, {'width': data}, self._cols).write(
            row, col, data)


class RowHeightWriter(BaseReportWriter):
    """
    Писатель высоты строки.
    """
    def write(self,
              row: int,
              col: int,
              data: Any
              ) -> None:
        RowWriter(self._report, {'height': data}).write(row, col, data)
